@@ -20,17 +20,6 @@
 * are permitted provided that all of the following conditions are met.
 * After December 31, 2001, only conditions 3-6 must be met:
 *
-* 1. Commercial and/or non-commercial use of the Software is permitted
-*    provided a current software registration is on file at www.OpenPBS.org.
-*    If use of this software contributes to a publication, product, or
-*    service, proper attribution must be given; see www.OpenPBS.org/credit.html
-*
-* 2. Redistribution in any form is only permitted for non-commercial,
-*    non-profit purposes.  There can be no charge for the Software or any
-*    software incorporating the Software.  Further, there can be no
-*    expectation of revenue generated as a consequence of redistributing
-*    the Software.
-*
 * 3. Any Redistribution of source code must retain the above copyright notice
 *    and the acknowledgment contained in paragraph 6, this list of conditions
 *    and the disclaimer contained in paragraph 7.
@@ -106,6 +95,7 @@
 
 #include "cmds.h"
 #include "mcom.h"
+#include "utils.h"
 
 static void states();
 
@@ -173,6 +163,7 @@ int isjobid(
 
 
 
+
 int istrue(
 
   char *string)  /* I */
@@ -352,6 +343,7 @@ void prt_attr(
 
 
 
+
 /*
  * locate a attribute (attrl) by name (and resource) and return value
  * returns null if not found
@@ -602,7 +594,6 @@ static void altdsp_statjob(
   char *jstate;
   char *eltimecpu;
   char *eltimewal;
-
   char tmpLine[MAX_LINE_LEN];
 
   int   usecput;
@@ -794,7 +785,6 @@ static void altdsp_statjob(
       }
     else
       {
-
       snprintf(tmpLine, sizeof(tmpLine), "%%-%d.%ds %%6.6s %%5.5s %%*.*s %%6.6s %%5.5s %%1.1s %%5.5s",
                PBS_NAMELEN, PBS_NAMELEN);
 
@@ -1046,7 +1036,7 @@ static void add_atropl(
 
 
 
-/* dispay when a normal "qstat" is executed */
+/* display when a normal "qstat" is executed */
 
 void display_statjob(
 
@@ -1186,6 +1176,7 @@ void display_statjob(
                 !strcmp(a->name, ATTR_qtime) ||
                 !strcmp(a->name, ATTR_start_time) ||
                 !strcmp(a->name, ATTR_comp_time) ||
+                !strcmp(a->name, ATTR_checkpoint_time) ||
                 !strcmp(a->name, ATTR_a))
               {
               epoch = (time_t)atoi(a->value);
@@ -1976,6 +1967,9 @@ void tcl_run(
 
 
 
+
+
+
 /* connects to server side routine pbs_statjob() in lib/Libifl/pbs_statjob.c */
 /*  routes to req_stat_job() in server/req_stat.c (PBS_BATCH_StatusJob) */
 
@@ -2504,31 +2498,32 @@ int main(
                 &queue_name_out,
                 &server_name_out))
             {
-            fprintf(stderr, "qstat: illegally formed destination: %s\n", destination);
+            fprintf(stderr, "qstat: illegally formed destination: %s\n",
+                    destination);
 
             ret = tcl_stat(error, NULL, f_opt);
+
             any_failed = 1;
+
             break;
             }
+
+          if (notNULL(server_name_out))
+            {
+            strcpy(server_out, server_name_out);
+            }
           else
             {
-            if (notNULL(server_name_out))
-              {
-              strcpy(server_out, server_name_out);
-              }
-            else
-              {
-              server_out[0] = '\0';
-              }
-
-            (void)strcpy(job_id_out, queue_name_out);
-
-            if (*queue_name_out != '\0')
-              {
-              add_atropl(&p_atropl, ATTR_q, NULL, queue_name_out, EQ);
-              }
+            server_out[0] = '\0';
             }
-          }
+
+          strcpy(job_id_out, queue_name_out);
+
+          if (*queue_name_out != '\0')
+            {
+            add_atropl(&p_atropl, ATTR_q, NULL, queue_name_out, EQ);
+            }
+          }    /* END else */
 
 job_no_args:
 
@@ -2583,13 +2578,16 @@ job_no_args:
             if (locate_job(job_id_out, server_out, rmt_server))
               {
               pbs_disconnect(connect);
+
               strcpy(server_out, rmt_server);
+
               goto job_no_args;
               }
 
             ret = tcl_stat("job", NULL, f_opt);
 
             prt_job_err("qstat", connect, job_id_out);
+
             any_failed = pbs_errno;
             }
           else
@@ -2599,6 +2597,7 @@ job_no_args:
             if (pbs_errno != PBSE_NONE)
               {
               prt_job_err("qstat", connect, job_id_out);
+
               any_failed = pbs_errno;
               }
             }
@@ -2609,8 +2608,10 @@ job_no_args:
             {
             altdsp_statjob(p_status, p_server, alt_opt);
             }
-          else if (f_opt == 0 || tcl_stat("job", p_status, f_opt))
+          else if ((f_opt == 0) || tcl_stat("job", p_status, f_opt))
+            {
             display_statjob(p_status, p_header, f_opt);
+            }
 
           p_header = FALSE;
 
@@ -2622,6 +2623,7 @@ job_no_args:
         break;
 
       case QUEUES:        /* get status of batch queues */
+
         strcpy(destination, operand);
 
         if (parse_destination_id(destination,
