@@ -110,6 +110,7 @@
 #define ATTR_a "Execution_Time"
 #define ATTR_c "Checkpoint"
 #define ATTR_e "Error_Path"
+#define ATTR_f "fault_tolerant"
 #define ATTR_g "group_list"
 #define ATTR_h "Hold_Types"
 #define ATTR_j "Join_Path"
@@ -165,28 +166,36 @@
 #define ATTR_exitstat   "exit_status"
 #define ATTR_forwardx11 "forward_x11"
 #define ATTR_submit_args "submit_args"
-#define ATTR_tokens      "tokens"
-#define ATTR_netcounter  "net_counter"
-#define ATTR_umask       "umask"
-#define ATTR_start_time  "start_time"
+#define ATTR_tokens     "tokens"
+#define ATTR_netcounter "net_counter"
+#define ATTR_umask      "umask"
+#define ATTR_start_time "start_time"
 #define ATTR_start_count "start_count"
-#define ATTR_chkptdir    "checkpoint_dir"    /* user specified directory for checkpoint files */
-#define ATTR_chkptname   "checkpoint_name"   /* user specified name of checkpoint file */
+#define ATTR_checkpoint_dir  "checkpoint_dir"    /* user specified directory for checkpoint files */
+#define ATTR_checkpoint_name  "checkpoint_name"   /* user specified name of checkpoint file */
+#define ATTR_checkpoint_time  "checkpoint_time"   /* timestamp of start of last checkpoint */
+#define ATTR_checkpoint_restart_status  "checkpoint_restart_status"   /* checkpoint restart status */
+#define ATTR_restart_name  "restart_name"   /* saved name of checkpoint restart file */
 #define ATTR_comp_time   "comp_time"
 #define ATTR_reported    "reported"
+#define ATTR_intcmd      "inter_cmd"
+
+#ifdef ENABLE_CSA
+#define ATTR_pagg         "pagg_id"
+#endif /* ENABLE_CSA */
 
 /* additional queue attributes names */
 
-#define ATTR_aclgren     "acl_group_enable"
-#define ATTR_aclgroup    "acl_groups"
-#define ATTR_aclhten     "acl_host_enable"
-#define ATTR_aclhost     "acl_hosts"
-#define ATTR_acluren     "acl_user_enable"
-#define ATTR_acluser     "acl_users"
-#define ATTR_altrouter   "alt_router"
-#define ATTR_chkptmin    "checkpoint_min"
-#define ATTR_enable      "enabled"
-#define ATTR_fromroute   "from_route_only"
+#define ATTR_aclgren    "acl_group_enable"
+#define ATTR_aclgroup   "acl_groups"
+#define ATTR_aclhten    "acl_host_enable"
+#define ATTR_aclhost    "acl_hosts"
+#define ATTR_acluren    "acl_user_enable"
+#define ATTR_acluser    "acl_users"
+#define ATTR_altrouter  "alt_router"
+#define ATTR_checkpoint_min  "checkpoint_min"
+#define ATTR_enable     "enabled"
+#define ATTR_fromroute  "from_route_only"
 #define ATTR_hostlist    "hostlist"         /* TORQUE only */
 #define ATTR_killdelay   "kill_delay"
 #define ATTR_maxgrprun   "max_group_run"
@@ -214,6 +223,7 @@
 #define ATTR_aclgrpslpy  "acl_group_sloppy"
 #define ATTR_keepcompleted "keep_completed"
 #define ATTR_disallowedtypes "disallowed_types"
+#define ATTR_is_transit  "is_transit"
 
 /* additional server attributes names */
 
@@ -265,25 +275,31 @@
 #define ATTR_servername  "server_name"
 #define ATTR_logfilemaxsize "log_file_max_size"
 #define ATTR_logfilerolldepth "log_file_roll_depth"
-#define ATTR_logkeepdays "log_keep_days"
-#define ATTR_nextjobnum  "next_job_number"
-#define ATTR_extraresc   "extra_resc"
+#define ATTR_logkeepdays  "log_keep_days"
+#define ATTR_nextjobnum   "next_job_number"
+#define ATTR_extraresc    "extra_resc"
 #define ATTR_schedversion "sched_version"
 #define ATTR_acctkeepdays "accounting_keep_days"
 #define ATTR_lockfile "lock_file"
 #define ATTR_credentiallifetime "credential_lifetime"
 #define ATTR_jobmustreport "job_must_report"
+#define ATTR_LockfileUpdateTime "lock_file_update_time"
+#define ATTR_LockfileCheckTime "lock_file_check_time"
+#define ATTR_npdefault  "np_default"
 
 /* additional node "attributes" names */
 
-#define ATTR_NODE_state      "state"
-#define ATTR_NODE_np         "np"
-#define ATTR_NODE_properties "properties"
-#define ATTR_NODE_ntype      "ntype"
-#define ATTR_NODE_jobs       "jobs"
-#define ATTR_NODE_status     "status"
-#define ATTR_NODE_note       "note"
+#define ATTR_NODE_state         "state"
+#define ATTR_NODE_np            "np"
+#define ATTR_NODE_properties    "properties"
+#define ATTR_NODE_ntype         "ntype"
+#define ATTR_NODE_jobs          "jobs"
+#define ATTR_NODE_status        "status"
+#define ATTR_NODE_note          "note"
 
+/* notification email formating */
+#define ATTR_mailsubjectfmt "mail_subject_fmt"
+#define ATTR_mailbodyfmt    "mail_body_fmt"
 
 
 /* various attribute values */
@@ -323,6 +339,9 @@
 #define Q_DT_interactive        "interactive"
 #define Q_DT_rerunable          "rerunable"
 #define Q_DT_nonrerunable       "nonrerunable"
+#define Q_DT_fault_tolerant     "fault_tolerant"
+#define Q_DT_fault_intolerant   "fault_intolerant"
+#define Q_DT_job_array          "job_array"
 
 /*constant related to sum of string lengths for above strings*/
 #define MAX_ENCODE_BFR  100
@@ -358,9 +377,10 @@
 /* WARNING: be careful changing these PBS_MAX* values.  They can result in a
    change in the structure of the .JB file, and binary incompatibilities between
    versions of TORQUE.  Do not change in a -fixes branch */
+   
+   
 
-
-#define PBS_MAXHOSTNAME  64 /* max host name length */
+#define PBS_MAXHOSTNAME  1024 /* max host name length */
 #ifndef MAXPATHLEN
 #define MAXPATHLEN  1024 /* max path name length */
 #endif
@@ -387,16 +407,14 @@
 #define PBS_TERM_BUF_SZ  80 /* Interactive term buffer size */
 #define PBS_TERM_CCA  6 /* Interactive term cntl char array */
 
-#define PBS_QS_VERSION  0x00020300 /* magic number used to determine version of pbs job quick save struct */
-/* the magic number is split into 4 8-bit chunks.  the first 8 bits are
-   unused.  the second b bits represent the major version number
-   third 8 bits are the minor version, and the final 8 bits are the
-   bug fix version.  we write the torque version that the job qs struct
-   was last changed in this constant writing them as if they are two
-   digit decimal numbers. all that really matters is that we assign a
-   unique value for each change in the ji_qs struct and that it can't
-   be confused with data at the start of a ji_qs struct from before
-   torque version 2.2.0 */
+/* constants used to indicate version of job ji_qs struct written to disk.
+   starting with 2.4.0 we stopped encoding the torque version number into the 
+   hex value e.g. 0x00020300 for torque v2.3.0. Now we just increment a value 
+   which is added to the last version encoded that way */
+#define PBS_QS_VERSION_BASE 0x00020300 /* magic number do not change */
+#define PBS_QS_VERSION_INT 1 /* increment this version number with every change to the ji_qs struct */
+#define PBS_QS_VERSION  (PBS_QS_VERSION_BASE + PBS_QS_VERSION_INT) /* version number saved in the ji_qs struct */
+
 /* someday the PBS_*_PORT definition will go away and only the */
 /* PBS_*_SERVICE_NAME form will be used, maybe   */
 
@@ -430,7 +448,15 @@
 #define PBS_SCHEDULER_SERVICE_PORT 15004
 #endif /* PBS_SCHEDULER_SERVICE_PORT */
 
-enum batch_op { SET, UNSET, INCR, DECR, EQ, NE, GE, GT, LE, LT, DFLT };
+/*
+ * Other misc checkpoint defines
+ */
+#define CHECKPOINTHOLD "checkpoint_hold"
+#define CHECKPOINTCONT "checkpoint_cont"
+#define MOM_DEFAULT_CHECKPOINT_DIR "$MOMDEFAULTCHECKPOINTDIR$"
+
+
+enum batch_op { SET, UNSET, INCR, DECR, EQ, NE, GE, GT, LE, LT, DFLT, MERGE };
 
 /*
 ** This structure is identical to attropl so they can be used
@@ -483,19 +509,19 @@ extern char *pbs_server; /* server attempted to connect | connected to */
 
 extern char *avail A_((int connect, char *resc));
 extern int pbs_asyrunjob A_((int c, char *jobid, char *location, char *extend));
+extern int pbs_alterjob_async A_((int connect, char *job_id, struct attrl *attrib, char *extend));
 extern int pbs_alterjob A_((int connect, char *job_id, struct attrl *attrib, char *extend));
 extern int pbs_connect A_((char *server));
 extern int pbs_query_max_connections();
 extern char *pbs_default A_((void));
 extern char *pbs_fbserver A_((void));
-extern int csv_length A_((char *str));
-extern char *csv_nth A_((char *str, int n));
 extern char *pbs_get_server_list A_((void));
 
 extern int pbs_deljob A_((int connect, char *job_id, char *extend));
 extern int pbs_disconnect A_((int connect));
 extern char *pbs_geterrmsg A_((int connect));
 extern int pbs_holdjob A_((int connect, char *job_id, char *hold_type, char *extend));
+extern int pbs_checkpointjob A_((int connect, char *job_id, char *extend));
 extern char *pbs_locjob A_((int connect, char *job_id, char *extend));
 
 extern int
