@@ -149,4 +149,8 @@ ATTR_acctkeepdays,
 ATTR_lockfile,
 ATTR_credentiallifetime,
 ATTR_jobmustreport,
-ATTR_jobstarttimeout,
+ATTR_checkpoint_dir,
+ATTR_mailsubjectfmt,
+ATTR_mailbodyfmt,
+ATTR_npdefault,
+
