@@ -92,6 +92,11 @@
  * the server attributes and resource (limits).
  */
 
+#ifndef SERVER_H
+#define SERVER_H
+
+#include "server_limits.h"
+
 enum srv_atr
   {
   SRV_ATR_State,                 /* 0 */
@@ -156,9 +161,14 @@ enum srv_atr
   SRV_ATR_SchedVersion,
   SRV_ATR_AcctKeepDays,          /* 60 */
   SRV_ATR_lockfile,
+  SRV_ATR_LockfileUpdateTime,
+  SRV_ATR_LockfileCheckTime,
   SRV_ATR_CredentialLifetime,
   SRV_ATR_JobMustReport,
-  SRV_ATR_JobStartTimeout,
+  SRV_ATR_checkpoint_dir,
+  SRV_ATR_MailSubjectFmt,
+  SRV_ATR_MailBodyFmt,
+  SRV_ATR_NPDefault,
 #include "site_svr_attr_enum.h"
   /* This must be last */
   SRV_ATR_LAST
@@ -250,4 +260,13 @@ typedef struct new_node
   char                *nn_name;   /* name of new node */
   } new_node;
 
+/* Threaded-HA code */
+#ifdef USE_HA_THREADS
+typedef pthread_mutex_t mutex_t;
+#else
+typedef int mutex_t;
+#endif /* USE_HA_THREADS */
+
 /* END server.h */
+
+#endif /* ifndef SERVER_H */
