@@ -89,10 +89,12 @@
 
 #define SET_LIMIT_SET   1
 #define SET_LIMIT_ALTER 0
-#define PBS_CHKPT_MIGRATE 0
-
+#define PBS_CHECKPOINT_MIGRATE 0
 
 
+#ifdef ENABLE_CSA
+#include <job.h>
+#endif /* ENABLE_CSA */
 
 
 /* struct startjob_rtn = used to pass error/session/other info  */
@@ -102,11 +104,15 @@ struct startjob_rtn
   {
   int   sj_code; /* error code */
   pid_t sj_session; /* session */
+
+#ifdef ENABLE_CSA
+  jid_t sj_csajobid;
+#endif /* ENABLE_CSA */
   };
 
 extern int mom_set_limits A_((job *, int)); /* Set job's limits */
 extern int mom_do_poll A_((job *));  /* Should limits be polled? */
-extern int mom_does_chkpnt();                   /* see if mom does chkpnt */
+extern int mom_does_checkpoint();                   /* see if mom does checkpoint */
 extern int mom_open_poll A_(());  /* Initialize poll ability */
 extern int mom_get_sample A_(());  /* Sample kernel poll data */
 extern int mom_over_limit A_((job *));  /* Is polled job over limit? */
