@@ -135,6 +135,7 @@ extern int mkdirtree(
     mode_t mode);
 
 extern int TTmpDirName(job*, char *);
+
 #endif /* HAVE_WORDEXP */
 
 /* External Global Data Items */
@@ -152,6 +153,7 @@ extern char            *msg_jobmod;
 extern char            *msg_manager;
 extern time_t  time_now;
 extern int  resc_access_perm; /* see encode_resc() */
+extern int spoolasfinalname;
 /* in attr_fn_resc.c */
 
 extern char             MOMUNameMissing[];
@@ -652,9 +654,9 @@ static int return_file(
 
     DIS_tcp_setup(sock);
 
-    if ((rc = tcp_encode_DIS_ReqHdr(sock, PBS_BATCH_MvJobFile, pbs_current_user)) ||
-        (rc = tcp_encode_DIS_JobFile(sock, seq++, buf, amt, pjob->ji_qs.ji_jobid, which)) ||
-        (rc = tcp_encode_DIS_ReqExtend(sock, NULL)))
+    if ((rc = encode_DIS_ReqHdr(sock, PBS_BATCH_MvJobFile, pbs_current_user)) ||
+        (rc = encode_DIS_JobFile(sock, seq++, buf, amt, pjob->ji_qs.ji_jobid, which)) ||
+        (rc = encode_DIS_ReqExtend(sock, NULL)))
       {
       break;
       }
@@ -780,6 +782,21 @@ static int told_to_cp(
       {
       if (wchost_match(host, pcphosts[nh].cph_hosts))
         {
+
+        if (LOGLEVEL >= 5)
+          {
+          sprintf(log_buffer, "host '%s' pcphosts[%d].cph_hosts: %s",
+                  host,
+                  nh,
+                  pcphosts[nh].cph_hosts);
+
+          log_record(
+            PBSEVENT_SYSTEM,
+            PBS_EVENTCLASS_SERVER,
+            (char *)id,
+            log_buffer);
+          }
+
         i = strlen(pcphosts[nh].cph_from);
 
         if (strncmp(pcphosts[nh].cph_from, oldpath, i) == 0)
@@ -2172,7 +2189,8 @@ void req_signaljob(
 
   numprocs = kill_job(pjob, sig, id, "killing job");
 
-  if ((numprocs == 0) && ((sig == 0)||(sig == SIGKILL)))
+  if ((numprocs == 0) && ((sig == 0)||(sig == SIGKILL)) &&
+    (pjob->ji_qs.ji_substate != JOB_SUBSTATE_OBIT))
     {
     /* SIGNUL and no procs found, force job to exiting */
     /* force issue of (another) job obit */
@@ -3141,6 +3159,14 @@ void req_cpyfile(
   int   wordexperr = 0;
 #endif
 
+  /* there is nothing to copy */
+  if (spoolasfinalname == TRUE)
+    {
+    reply_ack(preq);
+
+    return;
+    }
+
   if (LOGLEVEL >= 3)
     {
     pair = (struct rqfpair *)GET_NEXT(preq->rq_ind.rq_cpyfile.rq_pair);
@@ -3645,7 +3671,7 @@ void req_cpyfile(
           wordfree(&arg3exp);
 
           wordexperr = 0;
-
+          
           break; /* Successful */
           }
 
