@@ -20,17 +20,6 @@
 * are permitted provided that all of the following conditions are met.
 * After December 31, 2001, only conditions 3-6 must be met:
 *
-* 1. Commercial and/or non-commercial use of the Software is permitted
-*    provided a current software registration is on file at www.OpenPBS.org.
-*    If use of this software contributes to a publication, product, or
-*    service, proper attribution must be given; see www.OpenPBS.org/credit.html
-*
-* 2. Redistribution in any form is only permitted for non-commercial,
-*    non-profit purposes.  There can be no charge for the Software or any
-*    software incorporating the Software.  Further, there can be no
-*    expectation of revenue generated as a consequence of redistributing
-*    the Software.
-*
 * 3. Any Redistribution of source code must retain the above copyright notice
 *    and the acknowledgment contained in paragraph 6, this list of conditions
 *    and the disclaimer contained in paragraph 7.
@@ -108,7 +97,7 @@
 #include "server_limits.h"
 #include "server.h"
 #include "queue.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "resource.h"
 #include "work_task.h"
 #include "tracking.h"
@@ -119,6 +108,8 @@
 #include "pbs_proto.h"
 #include "batch_request.h"
 #include "array.h"
+#include "csv.h"
+#include "pbs_nodes.h"
 
 
 /*#ifndef SIGKILL*/
@@ -182,6 +173,7 @@ extern char *path_nodes_new;
 extern char *path_nodestate;
 extern char *path_nodenote;
 extern char *path_nodenote_new;
+extern char *path_checkpoint;
 
 extern int  queue_rank;
 extern char  server_name[];
@@ -237,9 +229,160 @@ static void  stop_me A_((int));
 #define CHANGE_STATE 1
 #define KEEP_STATE   0
 
+/**
+ * dynamic array, with utility functions for easy appending
+*/
+
+typedef struct darray_t {
+  int Length;
+  void **Data;
+  int AppendIndex;
+} darray_t;
+
+
+
+/**
+ * Initialize a dynamic array to a specific size
+ * @param Array (O) Assumed to be uninitialized struct
+ * @param InitialSize (I) raised to 0 if less than 0
+ */
+
+int DArrayInit(
+
+  darray_t *Array,      /* I */
+  int       InitialSize) /* I */
+
+  {
+  if (InitialSize <= 0)
+    {
+    Array->Length = 0;
+    Array->Data = NULL;
+    }
+  else
+    {
+    Array->Length = InitialSize;
+    Array->Data = (void **)malloc(sizeof(Array->Data[0]) * InitialSize);
+
+    if (Array->Data == NULL)
+      return(FAILURE);
+    }
+
+  Array->AppendIndex = 0;
+  return(SUCCESS);
+  } /*END DArrayInit */
+
+
+
+/**
+ * Free the resources associated with Array
+ * It does NOT free any data stored in the array, just the array structure itself.
+ * param Array (I)
+ */
+
+int DArrayFree(
+
+  darray_t *Array) /* I */
+
+  {
+  free(Array->Data);
+  Array->Data = NULL;
+  Array->Length = 0;
+  Array->AppendIndex = 0;
+  return(SUCCESS);
+  } /*END DArrayFree */
+
+
+
+/**
+ * Append Item onto the end of Array, resizing it if necessary 
+ * @param Array (I/O)
+ * @param Item (I)
+ */
+
+int DArrayAppend(
+
+  darray_t *Array, /* I/O */
+  void     *Item)  /* I */
+
+  {
+
+  if(Array->AppendIndex >= Array->Length)
+    {
+    int newLength = Array->Length * 2;
+
+    if(newLength <= 10)
+      newLength = 10;
+
+    Array->Length = newLength;
+    Array->Data = realloc(Array->Data,sizeof(Array->Data[0]) * Array->Length);
+
+    if(Array->Data == NULL)
+      {
+      Array->Length = 0;
+      Array->AppendIndex = 0;
+      return(FAILURE);
+      }
+    }
+
+  /*
+  assert(Array->AppendIndex >= 0);
+  assert(Array->AppendIndex < Array->Length);
+   */
+  Array->Data[Array->AppendIndex++] = Item;
+  return(SUCCESS);
+  } /* END DArrayAppend */
+
+
+
+/**
+ * Sort two job structs by their priority in ascending order
+ * @param A (I)
+ * @param B (I)
+ */
+
+static int SortPrioAscend(
+
+  const void *A, /* I */
+  const void *B) /* I */
+
+  {
+  job *pjob1 = *((job **)A);
+  job *pjob2 = *((job **)B);
+  int prio1 = pjob1->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
+  int prio2 = pjob2->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
+  return(prio1 - prio2);
+  } /*END SortPrioAscend */
+
+
+void  update_default_np()
+{
+  struct pbsnode *pnode;
+  int i;
+  long default_np;
+  long npfreediff;
+
+  default_np = server.sv_attr[(int)SRV_ATR_NPDefault].at_val.at_long;
+
+
+  if(default_np > 0)
+    {
+    for(i = 0; i < svr_totnodes; i++)
+      {
+      pnode = pbsndlist[i];
+
+       npfreediff = pnode->nd_nsn - pnode->nd_nsnfree;
+       pnode->nd_nsn = default_np;
+       pnode->nd_nsnfree = default_np - npfreediff;
+
+      }
+    }
+
+  return;
+}
+
 /* Add the server names from /var/spool/torque/server_name to the trusted hosts list. */
-void
-add_server_names_to_acl_hosts(void)
+
+void add_server_names_to_acl_hosts(void)
 
   {
   int n, list_len, rc;
@@ -275,9 +418,14 @@ add_server_names_to_acl_hosts(void)
       free_arst(&temp);
       }
     }
+
+  return;
   }
 
 
+
+
+
 /*
  * This file contains the functions to initialize the PBS Batch Server.
  * The code is called once when the server is brought up.
@@ -288,7 +436,7 @@ int pbsd_init(
   int type)  /* type of initialization   */
 
   {
-  int baselen;
+  int baselen = 0;
   char basen[MAXPATHLEN+1];
 
   struct dirent *pdirent;
@@ -306,6 +454,7 @@ int pbsd_init(
   pbs_queue *pque;
   char *psuffix;
   int  rc;
+  int Index;
 
   struct stat statbuf;
   char *suffix_slash = "/";
@@ -531,15 +680,11 @@ int pbsd_init(
 
   path_nodenote_new = build_path(path_priv, NODE_NOTE, new_tag);
 
+  path_checkpoint     = build_path(path_home, PBS_CHKPTDIR, suffix_slash);
+
   if (svr_resc_def == NULL)
     {
-    rc = init_resc_defs();
-    if (rc != 0)
-      {
-        log_err(rc, "pbsd_init", msg_init_baddb);
-
-        return(-1);
-      }
+    init_resc_defs();
     }
 
 #if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
@@ -710,6 +855,7 @@ int pbsd_init(
     }
 
   add_server_names_to_acl_hosts();
+  update_default_np();
 
   /*
    * 8. If not a "create" initialization, recover queues.
@@ -806,12 +952,14 @@ int pbsd_init(
 
   while ((pdirent = readdir(dir)) != NULL)
     {
+
     if (chk_save_file(pdirent->d_name) == 0)
       {
       /* if not create or clean recovery, recover arrays */
 
       if ((type != RECOV_CREATE) && (type != RECOV_COLD))
         {
+
         /* skip files without the proper suffix */
         baselen = strlen(pdirent->d_name) - array_suf_len;
 
@@ -820,28 +968,33 @@ int pbsd_init(
         if (strcmp(psuffix, ARRAY_FILE_SUFFIX))
           continue;
 
+
         pa = array_recov(pdirent->d_name);
 
         if (pa == NULL)
           {
           /* TODO GB */
 
-          sprintf(log_buffer, 
+          sprintf(log_buffer,
                   "could not recover array-struct from file %s--skipping",
                   pdirent->d_name);
 
           log_err(errno, "pbsd_init", log_buffer);
 
           continue;
+
           }
 
         pa->jobs_recovered = 0;
+
         }
       else
         {
         unlink(pdirent->d_name);
         }
+
       }
+
     }
 
   closedir(dir);
@@ -887,6 +1040,8 @@ int pbsd_init(
     }
   else
     {
+    darray_t Array;
+    DArrayInit(&Array,100);
     /* Now, for each job found ... */
 
     while ((pdirent = readdir(dir)) != NULL)
@@ -921,38 +1076,12 @@ int pbsd_init(
 
         if ((pjob = job_recov(pdirent->d_name)) != NULL)
           {
-          if (pbsd_init_job(pjob, type) == FAILURE)
+
+          if (DArrayAppend(&Array,pjob) == FAILURE)
             {
-            log_event(
-              PBSEVENT_ERROR | PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_JOB | PBSEVENT_FORCE,
-              PBS_EVENTCLASS_JOB,
-              pdirent->d_name,
-              msg_script_open);
+            log_err(ENOMEM,"main","out of memory reloading jobs");
+            exit(-1);
 
-            continue;
-            }
-
-          if ((type != RECOV_COLD) &&
-              (type != RECOV_CREATE) &&
-              (!(pjob->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)) &&
-              (pjob->ji_qs.ji_svrflags & JOB_SVFLG_SCRIPT))
-            {
-            strcpy(basen, pdirent->d_name);
-
-            psuffix = basen + baselen;
-
-            strcpy(psuffix, JOB_SCRIPT_SUFFIX);
-
-            if (chk_save_file(basen) != 0)
-              {
-              log_event(
-                PBSEVENT_ERROR | PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_JOB | PBSEVENT_FORCE,
-                PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid,
-                msg_script_open);
-
-              init_abt_job(pjob);
-              }
             }
           }
         else
@@ -980,9 +1109,49 @@ int pbsd_init(
             }
           }
         }
-      }
+      }    /* END while ((pdirent = readdir(dir)) != NULL) */
 
     closedir(dir);
+    qsort(Array.Data,Array.AppendIndex,sizeof(Array.Data[0]),SortPrioAscend);
+
+    for (Index = 0; Index < Array.AppendIndex; Index++)
+      {
+      job *pjob = (job *)Array.Data[Index];
+
+      if (pbsd_init_job(pjob, type) == FAILURE)
+        {
+        log_event(
+          PBSEVENT_ERROR | PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_JOB | PBSEVENT_FORCE,
+          PBS_EVENTCLASS_JOB,
+          pjob->ji_qs.ji_jobid,
+          msg_script_open);
+
+        continue;
+        }
+
+      if ((type != RECOV_COLD) &&
+          (type != RECOV_CREATE) &&
+          (!(pjob->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)) &&
+          (pjob->ji_qs.ji_svrflags & JOB_SVFLG_SCRIPT))
+        {
+        strcpy(basen, pjob->ji_qs.ji_jobid);
+
+        strcat(basen, JOB_SCRIPT_SUFFIX);
+
+        if (chk_save_file(basen) != 0)
+          {
+          log_event(
+            PBSEVENT_ERROR | PBSEVENT_SYSTEM | PBSEVENT_ADMIN | PBSEVENT_JOB | PBSEVENT_FORCE,
+            PBS_EVENTCLASS_JOB,
+            pjob->ji_qs.ji_jobid,
+            msg_script_open);
+
+          init_abt_job(pjob);
+          }
+        }
+      }
+
+    DArrayFree(&Array);
 
     if ((had != server.sv_qs.sv_numjobs) &&
         (type != RECOV_CREATE) &&
@@ -996,7 +1165,6 @@ int pbsd_init(
       }
 
     sprintf(log_buffer, msg_init_exptjobs,
-
             had, server.sv_qs.sv_numjobs);
 
     log_event(
@@ -1038,16 +1206,18 @@ int pbsd_init(
 
       if (pjob == NULL)
         {
-        /* TODO, we need to so something here, we can't finish cloning the array! */
+        /* TODO, we need to so something here, we can't finish cloning the 
+           array! */
 
         }
       else
         {
         /* TODO if num_cloned != num_recovered then something strange happend
-           it is possible num_recovered == num_cloned+1.  That means that the server
-           terminated after cloning a job but before updating the saved array_info struct.
-           we probably should delete that last job and start the cloning process off at
-           num_cloned */
+           it is possible num_recovered == num_cloned+1.  That means that the 
+           server terminated after cloning a job but before updating the saved 
+           array_info struct. we probably should delete that last job and start
+           the cloning process off at num_cloned. Someone must have been 
+           naughty and did a kill -9 on pbs_server  */
         wt = set_task(WORK_Timed, time_now + 1, job_clone_wt, (void*)pjob);
 
         }
@@ -1093,16 +1263,16 @@ int pbsd_init(
 
 #if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
 
-  if (chk_file_sec(path_track,0,0,S_IWGRP|S_IWOTH,0,EMsg) != 0)
+  if (chk_file_sec(path_track, 0, 0, S_IWGRP | S_IWOTH, 0, EMsg) != 0)
     {
     return(-1);
     }
 
 #endif  /* not DEBUG and not NO_SECURITY_CHECK */
 
-  if (fstat(fd,&statbuf) < 0)
+  if (fstat(fd, &statbuf) < 0)
     {
-    log_err(errno,"pbs_init","unable to stat tracking file");
+    log_err(errno, "pbs_init", "unable to stat tracking file");
 
     return(-1);
     }
@@ -1128,11 +1298,11 @@ int pbsd_init(
 
   /* NOTE:  tracking file records are optional */
 
-  i = read(fd,(char *)server.sv_track,server.sv_tracksize * sizeof(struct tracking));
+  i = read(fd, (char *)server.sv_track, server.sv_tracksize * sizeof(struct tracking));
 
   if (i < 0)
     {
-    log_err(errno,"pbs_init","unable to read tracksize from tracking file");
+    log_err(errno, "pbs_init", "unable to read tracksize from tracking file");
     }
 
   close(fd);
@@ -1141,7 +1311,7 @@ int pbsd_init(
 
   /* set work task to periodically save the tracking records */
 
-  set_task(WORK_Timed,(long)(time_now + PBS_SAVE_TRACK_TM),track_save,0);
+  set_task(WORK_Timed, (long)(time_now + PBS_SAVE_TRACK_TM), track_save, 0);
 
   /* SUCCESS */
 
@@ -1329,6 +1499,10 @@ static int pbsd_init_job(
 
     case JOB_SUBSTATE_STAGEGO:
 
+    case JOB_SUBSTATE_CHKPTGO:
+
+    case JOB_SUBSTATE_CHKPTCMP:
+
     case JOB_SUBSTATE_HELD:
 
     case JOB_SUBSTATE_SYNCHOLD:
@@ -1652,7 +1826,7 @@ static void catch_child(
         msg_daemonname,
         log_buffer);
 #else
-      DBPRT(("catch_child caught pid %d\n", (int)pid));
+      DBPRT(("catch_child caught pid %d\n", pid));
 #endif
       }
 
@@ -1683,7 +1857,7 @@ static void catch_child(
             msg_daemonname,
             log_buffer);
 #else
-          DBPRT(("catch_child found work task found for pid %d\n", (int)pid));
+          DBPRT(("catch_child found work task found for pid %d\n", pid));
 #endif
           }
 
@@ -1700,7 +1874,7 @@ static void catch_child(
 
       log_err(-1, "catch_child", log_buffer);
 #else
-      DBPRT(("catch_child no work task found for pid %d\n", (int)pid));
+      DBPRT(("catch_child no work task found for pid %d\n", pid));
 #endif
       }
     }    /* END while (1) */
@@ -1719,14 +1893,6 @@ void
 check_children(void)
 
   {
-  if (LOGLEVEL >= 7)
-    {
-    log_record(
-      PBSEVENT_SYSTEM | PBSEVENT_FORCE,
-      PBS_EVENTCLASS_SERVER,
-      msg_daemonname,
-      "check_children called");
-    }
 
   catch_child(0);
 
@@ -2092,13 +2258,7 @@ int get_svr_attr(
 
     if (svr_resc_def == NULL)
       {
-      rc = init_resc_defs();
-      if (rc != 0)
-        {
-        log_err(rc, "pbsd_init", msg_init_baddb);
-
-        return(-1);
-        }
+      init_resc_defs();
       }
 
     if (((rc = chk_save_file(path_svrdb))!= 0) || ((rc = svr_recov(path_svrdb, TRUE)) == -1)) 
