@@ -96,15 +96,17 @@
 #include "server.h"
 #include "credential.h"
 #include "batch_request.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "work_task.h"
 #include "pbs_error.h"
 #include "log.h"
 #include "acct.h"
 #include "svrfunc.h"
+#include "csv.h"
 
 /* Private Functions Local to this file */
 
+static void process_checkpoint_reply A_((struct work_task *));
 static void process_hold_reply A_((struct work_task *));
 
 /* Global Data Items: */
@@ -166,6 +168,7 @@ void req_holdjob(
   char    *pset;
   int     rc;
   attribute temphold;
+  attribute *pattr;
 
   pjob = chk_job_request(preq->rq_ind.rq_hold.rq_orig.rq_objname, preq);
 
@@ -199,7 +202,13 @@ void req_holdjob(
   sprintf(log_buffer, msg_jobholdset, pset, preq->rq_user,
           preq->rq_host);
 
-  if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
+  pattr = &pjob->ji_wattr[(int)JOB_ATR_checkpoint];
+
+  if ((pjob->ji_qs.ji_state == JOB_STATE_RUNNING) &&
+      ((pattr->at_flags & ATR_VFLAG_SET) &&
+       ((csv_find_string(pattr->at_val.at_str, "s") != NULL) ||
+        (csv_find_string(pattr->at_val.at_str, "c") != NULL) ||
+        (csv_find_string(pattr->at_val.at_str, "enabled") != NULL))))
     {
 
     /* have MOM attempt checkpointing */
@@ -212,14 +221,38 @@ void req_holdjob(
       }
     else
       {
-      pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
       pjob->ji_qs.ji_svrflags |=
-        JOB_SVFLG_HASRUN | JOB_SVFLG_CHKPT;
+        JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_FILE;
       job_save(pjob, SAVEJOB_QUICK);
+      
+      /* fill in log_buffer again, since relay_to_mom changed it */
+      
+      sprintf(log_buffer, msg_jobholdset, pset, preq->rq_user,
+          preq->rq_host);
+          
       LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid, log_buffer);
+        pjob->ji_qs.ji_jobid, log_buffer);
       }
     }
+#ifdef ENABLE_BLCR
+  else if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
+    {
+    /*
+     * This system is configured with BLCR checkpointing to be used,
+     * but this Running job does not have checkpointing enabled,
+     * so we reject the request
+     */
+
+    LOG_EVENT(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid,
+      log_buffer);
+
+    req_reject(PBSE_IVALREQ, 0, preq, NULL,
+        "job not held since checkpointing is expected but not enabled for job");
+    }
+#endif
   else
     {
     /* everything went well, may need to update the job state */
@@ -248,6 +281,65 @@ void req_holdjob(
 
 
 
+/*
+ * req_checkpointjob - service the Checkpoint Job Request
+ *
+ */
+
+void req_checkpointjob(
+
+  struct batch_request *preq)
+
+  {
+  job    *pjob;
+  int     rc;
+  attribute *pattr;
+
+  if ((pjob = chk_job_request(preq->rq_ind.rq_manager.rq_objname, preq)) == NULL)
+    {
+    return;
+    }
+
+  pattr = &pjob->ji_wattr[(int)JOB_ATR_checkpoint];
+
+  if ((pjob->ji_qs.ji_state == JOB_STATE_RUNNING) &&
+      ((pattr->at_flags & ATR_VFLAG_SET) &&
+       ((csv_find_string(pattr->at_val.at_str, "s") != NULL) ||
+        (csv_find_string(pattr->at_val.at_str, "c") != NULL) ||
+        (csv_find_string(pattr->at_val.at_str, "enabled") != NULL))))
+    {
+    /* have MOM attempt checkpointing */
+
+    if ((rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+                           preq, process_checkpoint_reply)) != 0)
+      {
+      req_reject(rc, 0, preq, NULL, NULL);
+      }
+    else
+      {
+      pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHECKPOINT_FILE;
+      job_save(pjob, SAVEJOB_QUICK);
+      LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
+                pjob->ji_qs.ji_jobid, log_buffer);
+      }
+    }
+  else
+    {
+    /* Job does not have checkpointing enabled, so reject the request */
+
+    LOG_EVENT(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_JOB,
+      pjob->ji_qs.ji_jobid,
+      log_buffer);
+
+    req_reject(PBSE_IVALREQ, 0, preq, NULL, "job is not checkpointable");
+    }
+  }  /* END req_checkpointjob() */
+
+
+
+
 /*
  * req_releasejob - service the Release Job Request
  *
@@ -409,6 +501,9 @@ static void process_hold_reply(
   struct batch_request *preq;
   int   newstate;
   int   newsub;
+  attribute temphold;
+  char *pset;
+  int rc;
 
   svr_disconnect(pwt->wt_event); /* close connection to MOM */
 
@@ -424,8 +519,18 @@ static void process_hold_reply(
     }
   else if (preq->rq_reply.brp_code != 0)
     {
+
+    rc = get_hold(&preq->rq_ind.rq_hold.rq_orig.rq_attr, &pset, &temphold);
+
+    if (rc == 0)
+      {
+      rc = job_attr_def[(int)JOB_ATR_hold].at_set(&pjob->ji_wattr[(int)JOB_ATR_hold],
+           &temphold, DECR);
+      }
+
     pjob->ji_qs.ji_substate = JOB_SUBSTATE_RUNNING;  /* reset it */
-    pjob->ji_modified = 1;    /* indicate attributes changed     */
+
+    pjob->ji_modified = 1;    /* indicate attributes changed */
     svr_evaljobstate(pjob, &newstate, &newsub, 0);
     svr_setjobstate(pjob, newstate, newsub); /* saves job */
 
@@ -445,17 +550,17 @@ static void process_hold_reply(
     {
     /* record that MOM has a checkpoint file */
 
-    /* Stupid PBS_CHKPT_MIGRATE is defined as zero therefore this code will never fire.
+    /* PBS_CHECKPOINT_MIGRATEABLE is defined as zero therefore this code will never fire.
      * And if these flags are not set, start_exec will not try to run the job from
      * the checkpoint image file.
      */
 
-    pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHKPT;
+    pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHECKPOINT_FILE;
 
-    if (preq->rq_reply.brp_auxcode)  /* chkpt can be moved */
+    if (preq->rq_reply.brp_auxcode)  /* checkpoint can be moved */
       {
-      pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_CHKPT;
-      pjob->ji_qs.ji_svrflags |=  JOB_SVFLG_HASRUN | JOB_SVFLG_ChkptMig;
+      pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_CHECKPOINT_FILE;
+      pjob->ji_qs.ji_svrflags |=  JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_MIGRATEABLE;
       }
 
     pjob->ji_modified = 1;    /* indicate attributes changed     */
@@ -463,7 +568,42 @@ static void process_hold_reply(
     svr_evaljobstate(pjob, &newstate, &newsub, 0);
     svr_setjobstate(pjob, newstate, newsub); /* saves job */
 
-    account_record(PBS_ACCT_CHKPNT, pjob, (char *)0);  /* note in accounting file */
+    account_record(PBS_ACCT_CHKPNT, pjob, "Checkpointed and held"); /* note in accounting file */
+    reply_ack(preq);
+    }
+  }
+
+/*
+ * process_checkpoint_reply
+ * called when a checkpoint request was sent to MOM and the answer
+ * is received.  Completes the checkpoint request for running jobs.
+ */
+
+static void process_checkpoint_reply(
+
+  struct work_task *pwt)
+  {
+  job       *pjob;
+
+  struct batch_request *preq;
+
+  svr_disconnect(pwt->wt_event); /* close connection to MOM */
+
+  preq = pwt->wt_parm1;
+  preq->rq_conn = preq->rq_orgconn;  /* restore client socket */
+
+  if ((pjob = find_job(preq->rq_ind.rq_manager.rq_objname)) == (job *)0)
+    {
+    LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
+              preq->rq_ind.rq_manager.rq_objname,
+              msg_postmomnojob);
+    req_reject(PBSE_UNKJOBID, 0, preq, NULL, msg_postmomnojob);
+    }
+  else
+    {
+    /* record that MOM has a checkpoint file */
+
+    account_record(PBS_ACCT_CHKPNT, pjob, "Checkpointed"); /* note in accounting file */
     reply_ack(preq);
     }
   }
