@@ -89,13 +89,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
 #include "libpbs.h"
 #include "server_limits.h"
 #include "list_link.h"
 #include "attribute.h"
 #include "resource.h"
 #include "server.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "credential.h"
 #include "batch_request.h"
 #include "work_task.h"
@@ -137,19 +138,17 @@ extern const char *PJobState[];
 /* External Functions called */
 
 extern void set_resc_assigned A_((job *, enum batch_op));
+extern void cleanup_restart_file(job *);
 
 /* Local public functions  */
 
 void req_jobobit A_((struct batch_request *));
 
-/* Local private functions */
-
-static struct batch_request *setup_cpyfiles A_((struct batch_request *, job *, char *, char *, int, int));
 
 
 /*
  * setup_from - setup the "from" name for a standard job file:
- * output, error, or chkpt
+ * output, error, or checkpoint
  */
 
 static char *setup_from(
@@ -180,7 +179,7 @@ static char *setup_from(
  * batch request, then append the file pairs
  */
 
-static struct batch_request *setup_cpyfiles(
+struct batch_request *setup_cpyfiles(
 
         struct batch_request *preq,
         job  *pjob,
@@ -231,9 +230,18 @@ static struct batch_request *setup_cpyfiles(
       pjob->ji_wattr[(int)JOB_ATR_job_owner].at_val.at_str,
       pcf->rq_owner);
 
-    get_jobowner(
-      pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str,
-      pcf->rq_user);
+    if (tflag == JOBCKPFILE)
+      {
+      get_jobowner(
+        PBS_DEFAULT_ADMIN,
+        pcf->rq_user);
+      }
+    else
+      {
+      get_jobowner(
+        pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str,
+        pcf->rq_user);
+      }
 
     if (((pjob->ji_wattr[(int)JOB_ATR_egroup].at_flags & ATR_VFLAG_DEFLT) == 0) &&
         (pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str != 0))
@@ -327,7 +335,49 @@ static int is_joined(
   }
 
 
+static struct batch_request *return_stdfile(
+        struct batch_request *preq,
+        job                  *pjob,
+        enum job_atr          ati)
 
+  {
+
+  if (pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags &&
+      pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
+    {
+    return NULL;
+    }
+
+  if ((pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET) == 0)
+    {
+    return NULL;
+    }
+
+
+  /* if this file is joined to another then it doesn't have to get copied back */
+  if (is_joined(pjob, ati))
+    {
+    return preq;
+    }
+
+  if (preq == NULL)
+    {
+    preq = alloc_br(PBS_BATCH_ReturnFiles);
+    }
+
+  strcpy(preq->rq_ind.rq_returnfiles.rq_jobid, pjob->ji_qs.ji_jobid);
+
+  if (ati == JOB_ATR_errpath)
+    {
+    preq->rq_ind.rq_returnfiles.rq_return_stderr = TRUE;
+    }
+  else if (ati == JOB_ATR_outpath)
+    {
+    preq->rq_ind.rq_returnfiles.rq_return_stdout = TRUE;
+    }
+
+  return preq;
+  }
 
 
 /*
@@ -392,8 +442,6 @@ static struct batch_request *cpy_stdfile(
 
   if (is_joined(pjob, ati))
     {
-    /* SUCCESS */
-
     return(preq);
     }
 
@@ -402,7 +450,6 @@ static struct batch_request *cpy_stdfile(
    * the keep list, MOM has already placed the file in the user's HOME
    * directory.  It doesn't need to be copied.
    */
-
   jkpattr = &pjob->ji_wattr[(int)JOB_ATR_keep];
 
   if ((jkpattr->at_flags & ATR_VFLAG_SET) &&
@@ -670,7 +717,6 @@ void rel_resc(
 
 
 
-
 /*
  * on_job_exit - continue post-execution processing of a job that terminated.
  *
@@ -697,7 +743,6 @@ void on_job_exit(
   {
   int    handle = -1;
   job   *pjob;
-  int    rc = 0;
 #ifdef VNODETESTING
   job   *pj;
 #endif
@@ -709,6 +754,10 @@ void on_job_exit(
   int  PurgeIt = FALSE;
   int MustReport = FALSE;
   pbs_queue *pque;
+  char namebuf[MAXPATHLEN + 1];
+  char *namebuf2;
+  int spool_file_exists;
+  int rc = 0;
 
   extern void remove_job_delete_nanny(struct job *);
 
@@ -805,12 +854,177 @@ void on_job_exit(
       svr_setjobstate(
         pjob,
         JOB_STATE_EXITING,
-        JOB_SUBSTATE_STAGEOUT);
+        JOB_SUBSTATE_RETURNSTD);
 
       ptask->wt_type = WORK_Immed;
 
       /* NO BREAK, fall into stage out processing */
 
+    case JOB_SUBSTATE_RETURNSTD:
+      /* this is a new substate to TORQUE 2.4.0.  The purpose is to provide a
+       * stage of the job exiting process that allows us to transfer stderr and
+       * stdout files from the mom's spool directory back to the server spool
+       * directory.  This will only be done if the job has been checkpointed,
+       * and keep_completed is a positive value. This is so that a completed
+       * job can be restarted from a checkpoint file.
+       */
+
+      if (ptask->wt_type != WORK_Deferred_Reply)
+        {
+        /* this is the very first call, have mom return the files */
+
+        /* if job has been checkpointed and KeepSeconds > 0, copy the stderr
+         * and stdout files back so that we can restart a completed
+         * checkpointed job return_stdfile will only setup this request if
+         * the job has a checkpoint file and the file is not joined to another
+         *  file */
+
+
+        KeepSeconds = 0;
+
+        if ((pque = pjob->ji_qhdr) && (pque->qu_attr != NULL))
+          {
+          KeepSeconds = attr_ifelse_long(
+                          &pque->qu_attr[(int)QE_ATR_KeepCompleted],
+                          &server.sv_attr[(int)SRV_ATR_KeepCompleted],
+                          0);
+          }
+
+        if (KeepSeconds > 0)
+          {
+
+
+          strcpy(namebuf, path_spool);
+          strcat(namebuf, pjob->ji_qs.ji_fileprefix);
+          strcat(namebuf, JOB_STDOUT_SUFFIX);
+
+          /* allocate space for the string name plus ".SAV" */
+          namebuf2 = malloc((strlen(namebuf) + 5) * sizeof(char));
+
+          if (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != pbs_server_addr)
+            {
+            preq = return_stdfile(preq, pjob, JOB_ATR_outpath);
+            }
+          else if (access(namebuf, F_OK) == 0)
+            {
+            strcpy(namebuf2, namebuf);
+            strcat(namebuf2, ".SAV");
+            link(namebuf, namebuf2);
+            }
+
+
+          namebuf[strlen(namebuf) - strlen(JOB_STDOUT_SUFFIX)] = '\0';
+
+          strcat(namebuf, JOB_STDERR_SUFFIX);
+
+
+          if (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != pbs_server_addr)
+            {
+            preq = return_stdfile(preq, pjob, JOB_ATR_errpath);
+            }
+          else if (access(namebuf, F_OK) == 0)
+            {
+            strcpy(namebuf2, namebuf);
+            strcat(namebuf2, ".SAV");
+            link(namebuf, namebuf2);
+            }
+
+          free(namebuf2);
+
+          }
+
+
+        if (preq != NULL)
+          {
+          preq->rq_extra = (void *)pjob;
+
+          if (issue_Drequest(handle, preq, on_job_exit, 0) == 0)
+            {
+            /* success, we'll come back after mom replies */
+            return;
+            }
+
+          /* set up as if mom returned error, if we fall through to
+           * here then we want to hit the error processing below
+           * because something bad happened */
+
+          IsFaked = 1;
+
+          preq->rq_reply.brp_code   = PBSE_MOMREJECT;
+
+          preq->rq_reply.brp_choice = BATCH_REPLY_CHOICE_NULL;
+
+          preq->rq_reply.brp_un.brp_txt.brp_txtlen = 0;
+
+
+          }
+        else
+          {
+          /* we don't need to return files to the server spool,
+             move on to see if we need to delete files */
+
+          svr_setjobstate(
+            pjob,
+            JOB_STATE_EXITING,
+            JOB_SUBSTATE_STAGEOUT);
+
+          if (LOGLEVEL >= 6)
+            {
+            log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              pjob->ji_qs.ji_jobid,
+              "no spool files to return");
+            }
+
+          ptask = set_task(WORK_Immed, 0, on_job_exit, pjob);
+
+          if (ptask != NULL)
+            {
+            append_link(&pjob->ji_svrtask, &ptask->wt_linkobj, ptask);
+            }
+
+          return;
+          }
+
+        }
+
+      /* here we have a reply (maybe faked) from MOM about the request */
+      if (preq->rq_reply.brp_code != 0)
+        {
+        if (LOGLEVEL >= 3)
+          {
+          snprintf(log_buffer, LOG_BUF_SIZE, "request to return spool files failed on node '%s' for job %s%s",
+                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_qs.ji_jobid,
+                   (IsFaked == 1) ? "*" : "");
+
+          log_event(
+            PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
+            PBS_EVENTCLASS_JOB,
+            pjob->ji_qs.ji_jobid,
+            log_buffer);
+          }
+        } /* end if (preq->rq_reply.brp_code != 0) */
+
+
+      /*
+       * files (generally) moved ok, move on to the next phase by
+       * "faking" the immediate work task and falling through to
+       * the next case.
+       */
+
+      free_br(preq);
+
+      preq = NULL;
+
+      svr_setjobstate(pjob, JOB_STATE_EXITING, JOB_SUBSTATE_STAGEOUT);
+
+      ptask->wt_type = WORK_Immed;
+
+
+      /* NO BREAK -- FALL INTO NEXT CASE */
+
     case JOB_SUBSTATE_STAGEOUT:
 
       if (LOGLEVEL >= 4)
@@ -853,8 +1067,9 @@ void on_job_exit(
 
           if (issue_Drequest(handle, preq, on_job_exit, 0) == 0)
             {
-            /* success--we will re-enter this function when MOM replies to
-             * our request */
+
+            /* request sucessfully sent, we'll come back to this function
+               when mom replies */
 
             return;
             }
@@ -884,7 +1099,7 @@ void on_job_exit(
           }
         else
           {
-          /* no files to copy, any to delete? */
+          /* no files to copy, go to next step */
 
           svr_setjobstate(
             pjob,
@@ -897,7 +1112,7 @@ void on_job_exit(
               PBSEVENT_JOB,
               PBS_EVENTCLASS_JOB,
               pjob->ji_qs.ji_jobid,
-              "no files to copy - deleting job");
+              "no files to copy");
             }
 
           ptask = set_task(WORK_Immed, 0, on_job_exit, pjob);
@@ -974,6 +1189,53 @@ void on_job_exit(
        * "faking" the immediate work task.
        */
 
+
+      /* check to see if we have saved the spool files, that means
+         the mom and server are sharing this spool directory.
+      pbs_server should take ownership of these files and rename them
+      see  JOB_SUBSTATE_RETURNSTD above*/
+
+      strcpy(namebuf, path_spool);
+
+      strcat(namebuf, pjob->ji_qs.ji_fileprefix);
+
+      strcat(namebuf, JOB_STDOUT_SUFFIX);
+
+      /* allocate space for the string name plus ".SAV" */
+      namebuf2 = malloc((strlen(namebuf) + 5) * sizeof(char));
+
+      strcpy(namebuf2, namebuf);
+
+      strcat(namebuf2, ".SAV");
+
+      spool_file_exists = access(namebuf2, F_OK);
+
+      if (spool_file_exists == 0)
+        {
+        link(namebuf2, namebuf);
+        unlink(namebuf2);
+        }
+
+
+
+      namebuf[strlen(namebuf) - strlen(JOB_STDOUT_SUFFIX)] = '\0';
+
+      strcat(namebuf, JOB_STDERR_SUFFIX);
+      strcpy(namebuf2, namebuf);
+      strcat(namebuf2, ".SAV");
+
+      spool_file_exists = access(namebuf2, F_OK);
+
+      if (spool_file_exists == 0)
+        {
+
+        link(namebuf2, namebuf);
+        unlink(namebuf2);
+        }
+
+      free(namebuf2);
+
+
       free_br(preq);
 
       preq = NULL;
@@ -984,6 +1246,7 @@ void on_job_exit(
 
       /* NO BREAK - FALL INTO THE NEXT CASE */
 
+
     case JOB_SUBSTATE_STAGEDEL:
 
       if (LOGLEVEL >= 4)
@@ -1017,7 +1280,7 @@ void on_job_exit(
 
           if (issue_Drequest(handle, preq, on_job_exit, 0) == 0)
             {
-            /* successful--we will re-enter this function when the MOM replies to this request */
+            /* request issued,  we'll come back when mom replies */
 
             return;
             }
@@ -1035,7 +1298,8 @@ void on_job_exit(
 
           IsFaked = 1;
 
-          /* set up as if mom returned error */
+          /* set up as if mom returned error since the issue_Drequest
+             failed */
 
           preq->rq_reply.brp_code = PBSE_MOMREJECT;
           preq->rq_reply.brp_choice = BATCH_REPLY_CHOICE_NULL;
@@ -1152,6 +1416,76 @@ void on_job_exit(
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         issue_track(pjob);
 
+      /* see if restarted job failed */
+
+      if (pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_flags & ATR_VFLAG_SET)
+        {
+        char *pfailtype = NULL;
+        char *pfailure = NULL;
+        long *hold_val = 0;
+        char errMsg[21];
+        
+        strncpy(errMsg,
+          pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str, 20);
+        
+        pfailtype = strtok(errMsg," ");
+        if (pfailtype != NULL)
+          pfailure = strtok(NULL," ");
+        
+        if (pfailure != NULL)
+          {
+          if (memcmp(pfailure,"failure",7) == 0)
+            {
+            if (memcmp(pfailtype,"Temporary",9) == 0)
+              {
+              /* reque job */
+ 
+              svr_setjobstate(pjob, JOB_STATE_QUEUED, JOB_SUBSTATE_QUEUED);
+              if (LOGLEVEL >= 4)
+                {
+                sprintf(log_buffer,
+                  "Requeueing job after checkpoint restart failure: %s",
+                  pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str);
+
+                log_event(
+                  PBSEVENT_JOB,
+                  PBS_EVENTCLASS_JOB,
+                  pjob->ji_qs.ji_jobid,
+                  log_buffer);
+                }
+              return;
+              }
+            /* 
+             * If we are deleting job after a failure then the first character
+             * of the comment should no longer be uppercase
+            */
+            else if (isupper(*pfailtype))
+              {
+              /* put job on hold */
+
+              hold_val = &pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long;
+              *hold_val |= HOLD_s;
+              pjob->ji_wattr[(int)JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
+              pjob->ji_modified = 1;
+              svr_setjobstate(pjob, JOB_STATE_HELD, JOB_SUBSTATE_HELD);
+              if (LOGLEVEL >= 4)
+                {
+                sprintf(log_buffer,
+                  "Placing job on hold after checkpoint restart failure: %s",
+                  pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str);
+
+                log_event(
+                  PBSEVENT_JOB,
+                  PBS_EVENTCLASS_JOB,
+                  pjob->ji_qs.ji_jobid,
+                  log_buffer);
+                }
+              return;
+              }
+            }
+          }     
+        }
+
       svr_setjobstate(pjob, JOB_STATE_COMPLETE, JOB_SUBSTATE_COMPLETE);
 
       if ((pque = pjob->ji_qhdr) && (pque != NULL))
@@ -1194,7 +1528,6 @@ void on_job_exit(
 
           pjob->ji_wattr[(int)JOB_ATR_reported].at_flags =
             ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
-
           job_save(pjob,SAVEJOB_FULL);
           }
         }
@@ -1266,7 +1599,6 @@ void on_job_exit(
          */
         
         PurgeIt = TRUE;
-
         if (((pjob->ji_wattr[(int)JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0)
           && (pjob->ji_wattr[(int)JOB_ATR_reported].at_val.at_long == 0))
           {
@@ -1719,15 +2051,6 @@ static void wait_for_send(
   struct work_task *ptask)
 
   {
-  if (LOGLEVEL >= 7)
-    {
-    log_event(
-      PBSEVENT_ERROR | PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      "",
-      "wait_for_send - job obit racing retry");
-    }
-
   req_jobobit((struct batch_request *)ptask->wt_parm1);
 
   return;
@@ -1793,17 +2116,7 @@ void req_jobobit(
   svrattrl  *patlist;
   unsigned int    dummy;
 
-  if (LOGLEVEL >= 7)
-    {
-    log_event(
-      PBSEVENT_ERROR | PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      preq->rq_ind.rq_jobobit.rq_jid,
-      "obit received");
-    }
-
   strcpy(jobid, preq->rq_ind.rq_jobobit.rq_jid);  /* This will be needed later for logging after preq is freed. */
-
   pjob = find_job(preq->rq_ind.rq_jobobit.rq_jid);
 
   if ((pjob == NULL) ||
@@ -1833,7 +2146,6 @@ void req_jobobit(
       }
 
     log_event(
-
       PBSEVENT_ERROR | PBSEVENT_JOB,
       PBS_EVENTCLASS_JOB,
       jobid,
@@ -1848,15 +2160,6 @@ void req_jobobit(
       {
       /* already in exit processing, ignore this request */
 
-      if (LOGLEVEL >= 7)
-        {
-        log_event(
-          PBSEVENT_ERROR | PBSEVENT_JOB,
-          PBS_EVENTCLASS_JOB,
-          preq->rq_ind.rq_jobobit.rq_jid,
-          "obit received - already in exit processing");
-        }
-
       bad = PBSE_ALRDYEXIT;
       }
     else
@@ -1898,28 +2201,8 @@ void req_jobobit(
 
     if (ptask == NULL)
       {
-      if (LOGLEVEL >= 7)
-        {
-        log_event(
-          PBSEVENT_ERROR | PBSEVENT_JOB,
-          PBS_EVENTCLASS_JOB,
-          preq->rq_ind.rq_jobobit.rq_jid,
-          "obit received - race condition and ptask is NULL");
-        }
-
       req_reject(PBSE_SYSTEM, 0, preq, NULL, NULL);
       }
-    else
-      {
-      if (LOGLEVEL >= 7)
-        {
-        log_event(
-          PBSEVENT_ERROR | PBSEVENT_JOB,
-          PBS_EVENTCLASS_JOB,
-          preq->rq_ind.rq_jobobit.rq_jid,
-          "obit received - race condition. Waiting for SIGCHLD, retry in 1 sec.");
-        }
-      }
 
     return;
     }
@@ -2028,6 +2311,7 @@ void req_jobobit(
 
     switch (exitstatus)
       {
+
       case JOB_EXEC_FAIL1:
 
       default:
@@ -2103,7 +2387,7 @@ void req_jobobit(
         /* MOM could not restart job, setup for rerun */
 
         alreadymailed = setrerun(pjob);
-        pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_CHKPT;
+        pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_CHECKPOINT_FILE;
 
         break;
 
@@ -2118,12 +2402,12 @@ void req_jobobit(
             PBSEVENT_JOB_USAGE | PBSEVENT_JOB_USAGE,
             PBS_EVENTCLASS_JOB,
             pjob->ji_qs.ji_jobid,
-            "received JOB_EXEC_INITRST, setting job CHKPT flag");
+            "received JOB_EXEC_INITRST, setting job CHECKPOINT_FLAG flag");
           }
 
         rel_resc(pjob);
 
-        pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_CHKPT;
+        pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_FILE;
 
         svr_evaljobstate(pjob, &newstate, &newsubst, 1);
 
@@ -2144,7 +2428,7 @@ void req_jobobit(
 
         alreadymailed = setrerun(pjob);
 
-        pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_ChkptMig;
+        pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_MIGRATEABLE;
 
         break;
       }  /* END switch (exitstatus) */
@@ -2226,17 +2510,25 @@ void req_jobobit(
         }
       }
 
+    /* remove checkpoint restart file if there is one */
+    
+    if (pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
+      {
+      cleanup_restart_file(pjob);
+      }
+
     /* "on_job_exit()" will be dispatched out of the main loop */
     }
   else
     {
+
     /* Rerunning job, if not checkpointed, clear "resources_used and requeue job */
 
-    if ((pjob->ji_qs.ji_svrflags & (JOB_SVFLG_CHKPT | JOB_SVFLG_ChkptMig)) == 0)
+    if ((pjob->ji_qs.ji_svrflags & (JOB_SVFLG_CHECKPOINT_FILE | JOB_SVFLG_CHECKPOINT_MIGRATEABLE)) == 0)
       {
       job_attr_def[(int)JOB_ATR_resc_used].at_free(&pjob->ji_wattr[(int)JOB_ATR_resc_used]);
       }
-    else if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_CHKPT)
+    else if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_CHECKPOINT_FILE)
       {
       /* non-migratable checkpoint (cray), leave there */
       /* and just requeue the job         */
@@ -2275,6 +2567,29 @@ void req_jobobit(
         }
       }
 
+#ifdef RERUNUSAGE
+
+    /* replace new-lines with blanks for accounting record */
+
+    for (pc = acctbuf;*pc;++pc)
+      {
+      if (*pc == '\n')
+        *pc = ' ';
+      }
+
+    /* record accounting  */
+
+    account_jobend(pjob, acctbuf);
+
+#endif    /* RERUNUSAGE */
+
+    /* remove checkpoint restart file if there is one */
+    
+    if (pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
+      {
+      cleanup_restart_file(pjob);
+      }
+
     /* "on_job_rerun()" will be dispatched out of the main loop */
     }  /* END else */
 
