@@ -121,7 +121,7 @@
 #include "pbs_error.h"
 #include "log.h"
 #include "svrfunc.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "pbs_nodes.h"
 #include "work_task.h"
 
@@ -167,8 +167,8 @@ static char *all_nodes = "_All_";
  * Returns: NULL if all ok or name of bad attribute if not ok
  */
 
-static char *check_que_attr(pque)
-pbs_queue  *pque;
+static char *
+check_que_attr(pbs_queue *pque)
   {
   int   i;
   int   type;
@@ -205,10 +205,12 @@ pbs_queue  *pque;
  * This is the at_action() routine for QA_ATR_Enabled
  */
 
-int check_que_enable(pattr, pque, mode)
-attribute *pattr;
-void      *pque; /* actually a queue* */
-int        mode;
+int
+check_que_enable(
+  attribute *pattr,
+  void *pque, /* actually a queue* */
+  int mode
+)
   {
   attribute *datr;
 
@@ -241,10 +243,8 @@ int        mode;
  * This is the at_action() routine for QA_ATR_QType
  */
 
-int set_queue_type(pattr, pque, mode)
-attribute *pattr;
-void      *pque;
-int    mode;
+int
+set_queue_type(attribute *pattr, void *pque, int mode)
   {
   int   i;
   char *pca;
@@ -2607,6 +2607,7 @@ int set_nextjobnum(
     case SET:
       attr->at_val.at_long = new->at_val.at_long;
       break;
+
       /* Code in Moab depends on this, do not change until the Moab code is fixed. */
 
     case INCR:
