@@ -110,12 +110,13 @@
 #include "server.h"
 #include "credential.h"
 #include "batch_request.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "queue.h"
 #include "net_connect.h"
 #include "pbs_error.h"
 #include "log.h"
 #include "svrfunc.h"
+#include "csv.h"
 
 #include "work_task.h"
 extern void  job_clone_wt A_((struct work_task *));
@@ -129,9 +130,7 @@ extern void start_exec A_((job *));
 extern int  svr_authorize_jobreq A_((struct batch_request *, job *));
 extern int  svr_chkque A_((job *, pbs_queue *, char *, int, char *));
 extern int  job_route A_((job *));
-#ifdef AUTORUN_JOBS
 extern int node_avail_complex(char *, int *, int *, int *, int*);
-#endif
 
 /* Global Data Items: */
 
@@ -152,7 +151,7 @@ const char *TJobFileType[] =
   "stdin",
   "stdout",
   "stderr",
-  "ckpt",
+  "checkpoint",
   NULL
   };
 
@@ -613,25 +612,25 @@ void req_quejob(
 
     /* need to set certain environmental variables per POSIX */
 
-    clear_attr(&tempattr,&job_attr_def[(int)JOB_ATR_variables]);
+    clear_attr(&tempattr, &job_attr_def[(int)JOB_ATR_variables]);
 
-    strcpy(buf,pbs_o_que);
+    strcpy(buf, pbs_o_que);
 
-    strcat(buf,pque->qu_qs.qu_name);
+    strcat(buf, pque->qu_qs.qu_name);
 
-    if (get_variable(pj,pbs_o_host) == NULL)
+    if (get_variable(pj, pbs_o_host) == NULL)
       {
-      strcat(buf,",");
-      strcat(buf,pbs_o_host);
-      strcat(buf,"=");
-      strcat(buf,preq->rq_host);
+      strcat(buf, ",");
+      strcat(buf, pbs_o_host);
+      strcat(buf, "=");
+      strcat(buf, preq->rq_host);
       }
 
     job_attr_def[(int)JOB_ATR_variables].at_decode(
-      &tempattr,
-      NULL,
-      NULL,
-      buf);
+        &tempattr,
+        NULL,
+        NULL,
+        buf);
 
     job_attr_def[(int)JOB_ATR_variables].at_set(
       &pj->ji_wattr[(int)JOB_ATR_variables],
@@ -642,46 +641,54 @@ void req_quejob(
 
     /* if JOB_ATR_outpath/JOB_ATR_errpath not set, set default */
 
-    if (!(pj->ji_wattr[(int)JOB_ATR_outpath].at_flags & ATR_VFLAG_SET))
+    if (!(pj->ji_wattr[(int)JOB_ATR_outpath].at_flags & ATR_VFLAG_SET) ||
+        (((pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str[strlen(pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str) - 1] == ':'))))
       {
       pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str =
         prefix_std_file(pj, (int)'o');
 
       pj->ji_wattr[(int)JOB_ATR_outpath].at_flags |= ATR_VFLAG_SET;
       }
-
-#if 0
-    else  /* change to show expected output path and filename */
+    /*
+     * if the output path was specified and ends with a '/'
+     * then append the standard file name
+     */
+    else if ((pj->ji_wattr[(int)JOB_ATR_outpath].at_flags & ATR_VFLAG_SET) &&
+        (((pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str[strlen(pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str) - 1] == '/'))))
       {
-      replace_attr_string(
-        &pj->ji_wattr[(int)JOB_ATR_outpath],
-        (add_std_filename(pj,
-        pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str,
-        (int)'o')));
+        pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str[strlen(pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str) - 1] = '\0';
+        
+        replace_attr_string(
+          &pj->ji_wattr[(int)JOB_ATR_outpath],
+          (add_std_filename(pj,
+          pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str,
+          (int)'o')));
       }
 
-#endif
-
-    if (!(pj->ji_wattr[(int)JOB_ATR_errpath].at_flags & ATR_VFLAG_SET))
+    if (!(pj->ji_wattr[(int)JOB_ATR_errpath].at_flags & ATR_VFLAG_SET) ||
+        (((pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str[strlen(pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str) - 1] == ':'))))
       {
       pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str =
         prefix_std_file(pj, (int)'e');
 
       pj->ji_wattr[(int)JOB_ATR_errpath].at_flags |= ATR_VFLAG_SET;
       }
-
-#if 0
-    else  /* change to show expected error path and filename */
+    /*
+     * if the error path was specified and ends with a '/'
+     * then append the standard file name
+     */
+    else if ((pj->ji_wattr[(int)JOB_ATR_errpath].at_flags & ATR_VFLAG_SET) &&
+        (((pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str[strlen(pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str) - 1] == '/'))))
       {
-      replace_attr_string(
-        &pj->ji_wattr[(int)JOB_ATR_errpath],
-        (add_std_filename(pj,
-        pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str,
-        (int)'e')));
+        pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str[strlen(pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str) - 1] = '\0';
+        
+        replace_attr_string(
+          &pj->ji_wattr[(int)JOB_ATR_errpath],
+          (add_std_filename(pj,
+          pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str,
+          (int)'e')));
       }
 
-#endif
-
     if ((pj->ji_wattr[(int)JOB_ATR_outpath].at_val.at_str == NULL) ||
         (pj->ji_wattr[(int)JOB_ATR_errpath].at_val.at_str == NULL))
       {
@@ -694,13 +701,42 @@ void req_quejob(
 
     /* If queue has checkpoint directory name specified, propagate it to the job. */
 
-    if (!(pj->ji_wattr[(int)JOB_ATR_chkptdir].at_flags & ATR_VFLAG_SET) &&
-        (pque->qu_attr[(int)QE_ATR_ChkptDir].at_flags & ATR_VFLAG_SET))
+    if (!(pj->ji_wattr[(int)JOB_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET))
       {
-      job_attr_def[(int)JOB_ATR_chkptdir].at_set(
-        &pj->ji_wattr[(int)JOB_ATR_chkptdir],
-        &pque->qu_attr[(int)QE_ATR_ChkptDir],
-        SET);
+      attribute *pattr;
+      char *vp;
+
+      pattr = &pj->ji_wattr[(int)JOB_ATR_checkpoint];
+
+      if ((pattr->at_flags & ATR_VFLAG_SET) &&
+          (vp = csv_find_value(pattr->at_val.at_str, "dir")))
+        {
+        clear_attr(
+          &pj->ji_wattr[(int)JOB_ATR_checkpoint_dir],
+          &job_attr_def[(int)JOB_ATR_checkpoint_dir]);
+
+        job_attr_def[(int)JOB_ATR_checkpoint_dir].at_decode(
+          &pj->ji_wattr[(int)JOB_ATR_checkpoint_dir],
+          NULL,
+          NULL,
+          vp);
+        }
+      else if ((pque->qu_attr[(int)QE_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET) &&
+               (pque->qu_attr[(int)QE_ATR_checkpoint_dir].at_val.at_str))
+        {
+        job_attr_def[(int)JOB_ATR_checkpoint_dir].at_set(
+          &pj->ji_wattr[(int)JOB_ATR_checkpoint_dir],
+          &pque->qu_attr[(int)QE_ATR_checkpoint_dir],
+          SET);
+        }
+      else if ((server.sv_attr[(int)SRV_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET) &&
+               (server.sv_attr[(int)SRV_ATR_checkpoint_dir].at_val.at_str))
+        {
+        job_attr_def[(int)JOB_ATR_checkpoint_dir].at_set(
+          &pj->ji_wattr[(int)JOB_ATR_checkpoint_dir],
+          &server.sv_attr[(int)SRV_ATR_checkpoint_dir],
+          SET);
+        }
       }
 
 #ifdef PNOT
@@ -722,7 +758,7 @@ void req_quejob(
         {
         job_purge(pj);
 
-        req_reject(PBSE_BADACCT,0,preq,NULL,"invalid account");
+        req_reject(PBSE_BADACCT, 0, preq, NULL, "invalid account");
 
         return;
         }
@@ -1020,7 +1056,7 @@ void req_jobscript(
   /* job has a script file */
 
   pj->ji_qs.ji_svrflags =
-    (pj->ji_qs.ji_svrflags & ~JOB_SVFLG_CHKPT) | JOB_SVFLG_SCRIPT;
+    (pj->ji_qs.ji_svrflags & ~JOB_SVFLG_CHECKPOINT_FILE) | JOB_SVFLG_SCRIPT;
 
   /* SUCCESS */
 
@@ -1087,9 +1123,9 @@ void req_mvjobfile(  /* NOTE:  routine for server only - mom code follows this r
 
       break;
 
-    case Chkpt:
+    case Checkpoint:
 
-      strcat(namebuf, JOB_CKPT_SUFFIX);
+      strcat(namebuf, JOB_CHECKPOINT_SUFFIX);
 
       break;
 
@@ -1321,11 +1357,11 @@ void req_commit(
   {
   job   *pj;
 
-#ifndef PBS_MOM /* SERVER only */
   int    newstate;
   int    newsub;
   pbs_queue *pque;
   int    rc;
+
 #ifdef AUTORUN_JOBS
 
   struct batch_request *preq_run = '\0';
@@ -1344,7 +1380,6 @@ void req_commit(
 
   char namebuf[MAXPATHLEN+1];
 #endif /* QUICKCOMMIT */
-#endif /* SERVER only */
 
   pj = locate_new_job(preq->rq_conn, preq->rq_ind.rq_commit);
 
@@ -1364,7 +1399,6 @@ void req_commit(
     return;
     }
 
-#ifndef PBS_MOM
 #ifdef QUICKCOMMIT
   if (pj->ji_qs.ji_substate != JOB_SUBSTATE_TRANSIN)
     {
@@ -1400,7 +1434,6 @@ void req_commit(
     }
 
 #endif /* QUICKCOMMIT */
-#endif /* SERVER only */
 
   if (pj->ji_qs.ji_substate != JOB_SUBSTATE_TRANSICM)
     {
@@ -1580,7 +1613,6 @@ void req_commit(
     issue_track(pj);
     }
 
-#ifndef PBS_MOM
 #ifdef AUTORUN_JOBS
   /* If we are auto running jobs with start_count = 0 then the
    * branch_request was re created. Now we run the job if any nodes
@@ -1604,7 +1636,6 @@ void req_commit(
     }
 
 #endif /* AUTORUN_JOBS */
-#endif /* PBS_MOM */
 
   return;
   }  /* END req_commit() */
