@@ -98,7 +98,7 @@
 #include "queue.h"
 #include "credential.h"
 #include "batch_request.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "work_task.h"
 #include "pbs_error.h"
 #include "svrfunc.h"
@@ -109,13 +109,13 @@
 /* Global Data Items: */
 
 extern struct server   server;
-extern tlist_head       svr_alljobs;
-extern tlist_head       svr_queues;
+extern tlist_head      svr_alljobs;
+extern tlist_head      svr_queues;
 extern char            server_name[];
 extern attribute_def   svr_attr_def[];
 extern attribute_def   que_attr_def[];
 extern attribute_def   job_attr_def[];
-extern attribute_def  node_attr_def[];   /* node attributes defs */
+extern attribute_def   node_attr_def[];   /* node attributes defs */
 extern int        pbs_mom_port;
 extern time_t        time_now;
 extern char       *msg_init_norerun;
@@ -131,6 +131,7 @@ extern int   svr_connect A_((pbs_net_t, unsigned int, void (*)(int), enum conn_t
 extern int status_nodeattrib(svrattrl *, attribute_def *, struct pbsnode *, int, int, tlist_head *, int*);
 extern int hasprop(struct pbsnode *, struct prop *);
 extern void rel_resc(job*);
+
 /* Private Data Definitions */
 
 static int bad;
@@ -147,7 +148,10 @@ static void stat_update A_((struct work_task *));
 #define TMAX_JOB 999999999
 #endif /* TMAX_JOB */
 
-/*
+
+
+
+/**
  * req_stat_job - service the Status Job Request
  *
  * This request processes the request for status of a single job or
@@ -287,15 +291,25 @@ void req_stat_job(
 
 /*
  * req_stat_job_step2 - continue with statusing of jobs
- * This is re-entered after sending status requests to MOM.
  *
- * Note, the funny initization/advance of pjob in the "while" loop
- * comes from the fact we want to look at the "next" job on re-entry.
+ *  This is re-entered after sending status requests to MOM.
+ *
+ *  NOTE:  because server job array and queue job arrays are basic linked
+ *         lists, this routine will report jobs in jobid order.
+ *
+ *         if truncated listed are desired, should handle this by walking
+ *         all queues and reporting max_report jobs/queue
+ *
+ *  Note, the funny initialization/advance of pjob in the "while" loop
+ *  comes from the fact we want to look at the "next" job on re-entry.
+ *
+ * @see req_stat_job() - parent
+ * @see status_job() - child - build job record
  */
 
 static void req_stat_job_step2(
 
-  struct stat_cntl *cntl)  /* I/O */
+  struct stat_cntl *cntl)  /* I/O (freed on return) */
 
   {
   svrattrl        *pal;
@@ -325,7 +339,42 @@ static void req_stat_job_step2(
   /* See pbs_server_attributes(1B) for details on "poll_jobs" behaviour */
 
   /* NOTE:  If IsTruncated is true, should walk all queues and walk jobs in each queue
-            until max_reported is reached */
+            until max_reported is reached (NYI) */
+
+  if (dpal == NULL)
+    {
+    /* build 'delta' attribute list */
+
+    svrattrl *tpal;
+
+    tlist_head dalist;
+
+    int aindex;
+
+    int atrlist[] =
+      {
+      JOB_ATR_jobname,
+      JOB_ATR_resc_used,
+      JOB_ATR_LAST
+      };
+
+    CLEAR_LINK(dalist);
+
+    for (aindex = 0;atrlist[aindex] != JOB_ATR_LAST;aindex++)
+      {
+      if ((tpal = attrlist_create("", "", 23)) == NULL)
+        {
+        return;
+        }
+
+      tpal->al_valln = atrlist[aindex];
+
+      if (dpal == NULL)
+        dpal = tpal;
+
+      append_link(&dalist, &tpal->al_link, tpal);
+      }
+    }  /* END if (dpal == NULL) */
 
   if (!server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long)
     {
@@ -358,7 +407,7 @@ static void req_stat_job_step2(
 
           pjob = (job *)GET_NEXT(svr_alljobs);
           }
-        }
+        }    /* END if (pjob == NULL) */
       else
         {
         /* get next job */
@@ -496,6 +545,7 @@ static void req_stat_job_step2(
         }
 
       sentJobCounter = 0;
+
       /* loop through jobs in queue */
 
       for (pjob = (job *)GET_NEXT(pque->qu_jobs);
@@ -551,7 +601,7 @@ static void req_stat_job_step2(
     return;
     }        /* END if ((type == tjstTruncatedServer) || ...) */
 
-  while (pjob != NULL) 
+  while (pjob != NULL)
     {
     /* go ahead and build the status reply for this job */
 
@@ -565,7 +615,12 @@ static void req_stat_job_step2(
 
     pal = (svrattrl *)GET_NEXT(preq->rq_ind.rq_status.rq_attr);
 
-    rc = status_job(pjob, preq, pal, &preply->brp_un.brp_status, &bad);
+    rc = status_job(
+           pjob,
+           preq,
+           pal,
+           &preply->brp_un.brp_status,
+           &bad);
 
     if ((rc != 0) && (rc != PBSE_PERM))
       {
@@ -632,18 +687,15 @@ nextjob:
 
 /*
  * stat_to_mom - send a Job Status to MOM to obtain latest status.
- * Used by req_stat_job()/stat_step_2().
- *
- * Frequency of this call is controlled by the job_stat_rate and poll_jobs
- * server attributes.
+ * Used by req_stat_job()/stat_step_2()
  *
  * Returns PBSE_SYSTEM if out of memory, PBSE_NORELYMOM if the MOM
  * is down, offline, or deleted.  Otherwise returns result of MOM
  * contact request.
+ *
+ * NOTE: called by qstat if poll_jobs == False
  */
 
-/* NOTE:  called by qstat */
-
 int stat_to_mom(
 
   job              *pjob,  /* I */
@@ -807,8 +859,6 @@ static void stat_update(
       }
     }
 
-
-
   release_req(pwt);
 
   cntl->sc_conn = -1;
@@ -865,7 +915,13 @@ void stat_mom_job(
 
 
 
-
+/**
+ * poll_job_task
+ *
+ * The invocation of this routine is triggered from
+ * the pbs_server main_loop code.  The check of
+ * SRV_ATR_PollJobs appears to be redundant.
+ */
 void poll_job_task(
 
   struct work_task *ptask)
@@ -995,9 +1051,9 @@ void req_stat_que(
 
 static int status_que(
 
-  pbs_queue *pque, /* ptr to que to status */
+  pbs_queue            *pque,     /* ptr to que to status */
   struct batch_request *preq,
-  tlist_head *pstathd) /* head of list to append status to */
+  tlist_head           *pstathd)  /* head of list to append status to */
 
   {
 
@@ -1255,10 +1311,7 @@ static int status_node(
 
   bad = 0;                                    /*global variable*/
 
-  if (preq->rq_ind.rq_status.rq_attr.ll_struct != NULL)
-    pal = (svrattrl *)GET_NEXT(preq->rq_ind.rq_status.rq_attr);
-  else
-    pal = NULL;
+  pal = (svrattrl *)GET_NEXT(preq->rq_ind.rq_status.rq_attr);
 
   rc = status_nodeattrib(
          pal,
@@ -1373,10 +1426,8 @@ void req_stat_svr(
  * attributes.
  */
 
-static void update_state_ct(pattr, ct_array, buf)
-attribute     *pattr;
-int       *ct_array;
-char       *buf;
+static void
+update_state_ct(attribute *pattr, int *ct_array, char *buf)
   {
   static char *statename[] = { "Transit", "Queued", "Held",
                                "Waiting", "Running", "Exiting"
