@@ -114,6 +114,7 @@ extern int set_tokens A_((struct attribute *attr, struct attribute *new, enum ba
 
 extern int extra_resc_chk A_((attribute *pattr, void *pobject, int actmode));
 extern void free_extraresc A_((attribute *attr));
+extern void restore_attr_default A_((struct attribute *));
 
 /* DIAGTODO: write diag_attr_def.c */
 
@@ -339,7 +340,7 @@ attribute_def svr_attr_def[] =
     encode_l,
     set_l,
     comp_l,
-    free_noop,  /* disable unset */
+    restore_attr_default,
     NULL_FUNC,
     NO_USER_SET,
     ATR_TYPE_LONG,
@@ -495,7 +496,7 @@ attribute_def svr_attr_def[] =
       encode_l,
       set_l,
       comp_l,
-      free_noop,  /* disable unset */
+      restore_attr_default,
       NULL_FUNC,
       NO_USER_SET,
       ATR_TYPE_LONG,
@@ -560,7 +561,7 @@ attribute_def svr_attr_def[] =
     encode_l,
     set_l,
     comp_l,
-    free_noop,  /* disable unset */
+    restore_attr_default,
     NULL_FUNC,
     MGR_ONLY_SET,
     ATR_TYPE_LONG,
@@ -573,7 +574,7 @@ attribute_def svr_attr_def[] =
     encode_b,
     set_b,
     comp_b,
-    free_noop,  /* disable unset */
+    restore_attr_default,
     NULL_FUNC,
     MGR_ONLY_SET,
     ATR_TYPE_LONG,
@@ -586,7 +587,7 @@ attribute_def svr_attr_def[] =
       encode_l,
       set_l,
       comp_l,
-      free_noop,  /* disable unset */
+      restore_attr_default,
       NULL_FUNC,
       MGR_ONLY_SET,
       ATR_TYPE_LONG,
@@ -841,7 +842,7 @@ attribute_def svr_attr_def[] =
       PARENT_TYPE_SERVER
   },
 
-  /* SVR_ATR_NextJobNumber */
+  /* SRV_ATR_NextJobNumber */
   { ATTR_nextjobnum,
     decode_l,
     encode_l,
@@ -945,6 +946,32 @@ attribute_def svr_attr_def[] =
     	PARENT_TYPE_SERVER
   },
 
+  /* SRV_ATR_lockfile */
+  {   ATTR_LockfileUpdateTime, /* lock_file_update_time */
+      decode_l,
+      encode_l,
+      set_l,
+      comp_l,
+      free_null,
+      NULL_FUNC,
+      MGR_ONLY_SET,
+      ATR_TYPE_STR,
+      PARENT_TYPE_SERVER
+  },
+
+  /* SRV_ATR_LockfileCheckTime */
+  {   ATTR_LockfileCheckTime, /* lock_file_check_time */
+      decode_l,
+      encode_l,
+      set_l,
+      comp_l,
+      free_null,
+      NULL_FUNC,
+      MGR_ONLY_SET,
+      ATR_TYPE_STR,
+      PARENT_TYPE_SERVER
+  },
+
   /* SRV_ATR_CredentialLifetime */
   {   ATTR_credentiallifetime,  /* "credential_lifetime" */
       decode_l,
@@ -971,20 +998,57 @@ attribute_def svr_attr_def[] =
     	PARENT_TYPE_SERVER
   },
 
-  /* SRV_ATR_JobStartTimeout */
-  {   ATTR_jobstarttimeout,         /* "job_start_timeout" */
+  /* SRV_ATR_checkpoint_dir */
+  {   ATTR_checkpoint_dir,   /* "checkpoint_dir" */
+      decode_str,
+      encode_str,
+      set_str,
+      comp_str,
+      free_str,
+      NULL_FUNC,
+      MGR_ONLY_SET,
+      ATR_TYPE_STR,
+      PARENT_TYPE_SERVER
+  },
+
+  /* SRV_ATR_MailSubjectFmt */
+  { ATTR_mailsubjectfmt, /* "mail_subject_fmt" */
+    decode_str,
+    encode_str,
+    set_str,
+    comp_str,
+    free_str,
+    NULL_FUNC,
+    MGR_ONLY_SET,
+    ATR_TYPE_STR,
+    PARENT_TYPE_SERVER
+  },
+
+  /* SRV_ATR_MailBodyFmt */
+  { ATTR_mailbodyfmt, /* "mail_body_fmt" */
+    decode_str,
+    encode_str,
+    set_str,
+    comp_str,
+    free_str,
+    NULL_FUNC,
+    MGR_ONLY_SET,
+    ATR_TYPE_STR,
+    PARENT_TYPE_SERVER
+  },
+    /* SRV_ATR_NPDefault */
+  {   ATTR_npdefault,          /* "np_default" */
       decode_l,
       encode_l,
       set_l,
       comp_l,
       free_null,
       NULL_FUNC,
-      NO_USER_SET,
+      MGR_ONLY_SET,
       ATR_TYPE_LONG,
       PARENT_TYPE_SERVER
   },
 
-
   /* site supplied server attribute definitions if any, see site_svr_attr_*.h  */
 #include "site_svr_attr_def.h"
 
