@@ -92,12 +92,13 @@
 #include "list_link.h"
 #include "attribute.h"
 #include "resource.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "queue.h"
 #include "server.h"
 #include "pbs_error.h"
 #include "svrfunc.h"
 #include "sched_cmds.h"
+#include "csv.h"
 
 extern int    scheduler_sock;
 extern int    svr_do_schedule;
@@ -119,6 +120,10 @@ static char *svr_state_names[] =
   "Terminating"  /* SV_STATE_SHUTSIG */
   };
 
+
+
+
+
 /*
  * encode_svrstate - encode the current server state from the internal
  * integer to a state name string.
@@ -126,11 +131,11 @@ static char *svr_state_names[] =
 
 int encode_svrstate(
 
-  attribute       *pattr,    /* ptr to attribute */
-  tlist_head       *phead,   /* head of attrlist list */
+  attribute       *pattr,   /* ptr to attribute */
+  tlist_head      *phead,   /* head of attrlist list */
   char            *atname,  /* attribute name */
   char            *rsname,  /* null */
-  int             mode)     /* encode mode */
+  int              mode)    /* encode mode */
 
   {
   svrattrl *pal;
@@ -304,10 +309,14 @@ void set_resc_assigned(
 
 
 /*
- * ck_chkpnt - check validity of job checkpoint attribute value
+ * ck_checkpoint - check validity of job checkpoint attribute value
+ *
+ * This routine is not directly called.
+ * Rather it is referenced by an at_action field.
+ * This is invoked inside of a loop over attributes in req_quejob.
  */
 
-int ck_chkpnt(
+int ck_checkpoint(
 
   attribute *pattr,
   void      *pobject, /* not used */
@@ -315,6 +324,10 @@ int ck_chkpnt(
 
   {
   char *val;
+  int field_count;
+  int i;
+  int len;
+  char *str;
 
   val = pattr->at_val.at_str;
 
@@ -325,32 +338,43 @@ int ck_chkpnt(
     return(0);
     }
 
-  if ((*val == 'n') || (*val == 's') || (*val == 'u'))
-    {
-    if (*(val + 1) != '\0')
-      {
-      return(PBSE_BADATVAL);
-      }
-    }
-  else if (*val++ == 'c')
-    {
-    if (*val != '\0')
-      {
-      if (*val++ != '=')
-        {
-        return(PBSE_BADATVAL);
-        }
+  field_count = csv_length(val);
 
-      if (atoi(val) <= 0)
+  for (i = 0; i < field_count; i++)
+    {
+    str = csv_nth(val, i);
+
+    if (str)
+      {
+      if ((len = strlen(str)) > 0)
         {
-        return(PBSE_BADATVAL);
+        if (len == 1 && !((str[0] == 'n') || (str[0] == 's') || (str[0] == 'u')))
+          {
+          return(PBSE_BADATVAL);
+          }
+        else if (!strncmp(str, "c=", 2))
+          {
+          if (atoi(&str[2]) <= 0)
+            {
+            return(PBSE_BADATVAL);
+            }
+
+          continue;
+          }
+        else if (!strcmp(str, "none"))            {}
+        else if (!strcmp(str, "periodic"))        {}
+        else if (!strcmp(str, "shutdown"))        {}
+        else if (!strcmp(str, "enabled"))       {}
+        else if (!strncmp(str, "interval=", 9))    {}
+        else if (!strncmp(str, "depth=", 6))       {}
+        else if (!strncmp(str, "dir=", 4))         {}
+        else
+          {
+          return(PBSE_BADATVAL);
+          }
         }
       }
     }
-  else
-    {
-    return(PBSE_BADATVAL);
-    }
 
   /* SUCCESS */
 
@@ -366,11 +390,8 @@ int ck_chkpnt(
  * and queue ) attributes.  It just returns 0.
  */
 
-int decode_null(patr, name, rn, val)
-attribute *patr;
-char      *name;
-char      *rn;
-char      *val;
+int
+decode_null(attribute *patr, char *name, char *rn, char *val)
   {
   return 0;
   }
